﻿using System;
using System.IO;
using System.Text;
using System.Security.Cryptography;

namespace TestProject
{
    /// <summary>
    /// AES 헬퍼
    /// </summary>
    public static class AESHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Field

        #region Field

        /// <summary>
        /// SALT 배열
        /// </summary>
        private static byte[] _saltArray = Encoding.ASCII.GetBytes("123456789abcdefghijkllmn");

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 암호화 하기 - Encrypt(sourceText, key)

        /// <summary>
        /// 암호화 하기
        /// </summary>
        /// <param name="sourceText">소스 텍스트</param>
        /// <param name="key">키</param>
        /// <returns>암호화 텍스트</returns>
        public static string Encrypt(string sourceText, string key)
        {
            string          targetText      = null;
            RijndaelManaged rijndaelManaged = null;

            try
            {
                Rfc2898DeriveBytes rfc2898DeriveBytes = new Rfc2898DeriveBytes(key, _saltArray);

                rijndaelManaged = new RijndaelManaged();

                rijndaelManaged.Key = rfc2898DeriveBytes.GetBytes(rijndaelManaged.KeySize / 8);

                ICryptoTransform cryptoTransform = rijndaelManaged.CreateEncryptor(rijndaelManaged.Key, rijndaelManaged.IV);

                using(MemoryStream memoryStream = new MemoryStream())
                {
                    memoryStream.Write(BitConverter.GetBytes(rijndaelManaged.IV.Length), 0, sizeof(int));

                    memoryStream.Write(rijndaelManaged.IV, 0, rijndaelManaged.IV.Length);

                    using(CryptoStream cryptoStream = new CryptoStream(memoryStream, cryptoTransform, CryptoStreamMode.Write))
                    {
                        using(StreamWriter streamWriter = new StreamWriter(cryptoStream))
                        {
                            streamWriter.Write(sourceText);
                        }
                    }

                    targetText = Convert.ToBase64String(memoryStream.ToArray());
                }
            }
            finally
            {
                if(rijndaelManaged != null)
                {
                    rijndaelManaged.Clear();
                }
            }

            return targetText;
        }

        #endregion
        #region 복호화 하기 - Decrypt(sourceText, key)

        /// <summary>
        /// 복호화 하기
        /// </summary>
        /// <param name="sourceText">소스 텍스트</param>
        /// <param name="key">키</param>
        /// <returns>복호화 텍스트</returns>
        public static string Decrypt(string sourceText, string key)
        {
            RijndaelManaged rijndaelManaged = null;
            string          targetText      = null;

            try
            {
                Rfc2898DeriveBytes rfc2898DeriveBytes = new Rfc2898DeriveBytes(key, _saltArray);

                byte[] sourceByteArray = Convert.FromBase64String(sourceText);

                using(MemoryStream memoryStream = new MemoryStream(sourceByteArray))
                {
                    rijndaelManaged = new RijndaelManaged();

                    rijndaelManaged.Key = rfc2898DeriveBytes.GetBytes(rijndaelManaged.KeySize / 8);
                    rijndaelManaged.IV  = ReadByteArray(memoryStream);

                    ICryptoTransform dryptoTransform = rijndaelManaged.CreateDecryptor(rijndaelManaged.Key, rijndaelManaged.IV);

                    using(CryptoStream cryptoStream = new CryptoStream(memoryStream, dryptoTransform, CryptoStreamMode.Read))
                    {
                        using(StreamReader streamReader = new StreamReader(cryptoStream))
                        {
                            targetText = streamReader.ReadToEnd();
                        }
                    }
                }
            }
            finally
            {
                if(rijndaelManaged != null)
                {
                    rijndaelManaged.Clear();
                }
            }

            return targetText;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 바이트 배열 읽기 - ReadByteArray(stream)

        /// <summary>
        /// 바이트 배열 읽기
        /// </summary>
        /// <param name="stream">스트림</param>
        /// <returns>바이트 배열</returns>
        private static byte[] ReadByteArray(Stream stream)
        {
            byte[] rawLengthArray = new byte[sizeof(int)];

            if(stream.Read(rawLengthArray, 0, rawLengthArray.Length) != rawLengthArray.Length)
            {
                throw new SystemException();
            }

            byte[] bufferArray = new byte[BitConverter.ToInt32(rawLengthArray, 0)];

            if(stream.Read(bufferArray, 0, bufferArray.Length) != bufferArray.Length)
            {
                throw new SystemException();
            }

            return bufferArray;
        }

        #endregion
    }
}