﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 소스 비트맵
        /// </summary>
        private Bitmap sourceBitmap = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 열기 메뉴 항목 클릭시 처리하기 - openMenuItem_Click(sender, e)

        /// <summary>
        /// 열기 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void openMenuItem_Click(object sender, EventArgs e)
        {
            if(this.openFileDialog.ShowDialog() == DialogResult.OK)
            {
                this.pictureBox.Image = Image.FromFile(this.openFileDialog.FileName);

                this.encryptButton.Enabled = true;
                this.decryptButton.Enabled = true;
            }
        }

        #endregion
        #region 종료 메뉴 항목 클릭시 처리하기 - exitMenuItem_Click(sender, e)

        /// <summary>
        /// 종료 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void exitMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        #endregion

        #region 암호화 버튼 클릭시 처리하기 - encryptButton_Click(sender, e)

        /// <summary>
        /// 암호화 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void encryptButton_Click(object sender, EventArgs e)
        {
            if(this.pictureBox.Image == null)
            {
                MessageBox.Show(this, "이미지 파일을 선택해 주시기 바랍니다.", "INFORMATION", MessageBoxButtons.OK, MessageBoxIcon.Information);

                return;
            }

            this.sourceBitmap = this.pictureBox.Image as Bitmap;

            string sourceText = this.textBox.Text;

            if(string.IsNullOrWhiteSpace(sourceText))
            {
                MessageBox.Show(this, "텍스트를 입력해 주시기 바랍니다.", "INFORMATION", MessageBoxButtons.OK, MessageBoxIcon.Information);

                this.textBox.Focus();

                return;
            }

            if(this.keyTextBox.Text.Length < 8)
            {
                MessageBox.Show(this, "비밀키는 8자리 이상 입력해 주시기 바랍니다.", "INFORMATION", MessageBoxButtons.OK, MessageBoxIcon.Information);

                this.keyTextBox.Focus();

                return;
            }
            else
            {
                sourceText = AESHelper.Encrypt(sourceText, this.keyTextBox.Text);
            }

            sourceBitmap = SteganographyHelper.GetBitmap(sourceText, sourceBitmap);

            if(this.saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                switch(this.saveFileDialog.FilterIndex)
                {
                    case 0 :
                    {
                        this.sourceBitmap.Save(this.saveFileDialog.FileName, ImageFormat.Png);

                        break;
                    }
                    case 1 :
                    {
                        this.sourceBitmap.Save(this.saveFileDialog.FileName, ImageFormat.Bmp);

                        break;
                    }
                }
            }

            ClearControlData();
        }

        #endregion
        #region 복호화 버튼 클릭시 처리하기 - decryptButton_Click(sender, e)

        /// <summary>
        /// 복호화 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void decryptButton_Click(object sender, EventArgs e)
        {
            this.sourceBitmap = this.pictureBox.Image as Bitmap;

            string text = SteganographyHelper.GetText(sourceBitmap);

            try
            {
                text = AESHelper.Decrypt(text, this.keyTextBox.Text);
            }
            catch
            {
                MessageBox.Show(this, "비밀키가 일치하지 않습니다.", "INFORMATION", MessageBoxButtons.OK, MessageBoxIcon.Information);

                this.keyTextBox.Focus();

                return;
            }

            this.textBox.Text = text;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 컨트롤 데이터 지우기 - ClearControlData()

        /// <summary>
        /// 컨트롤 데이터 지우기
        /// </summary>
        private void ClearControlData()
        {
            this.pictureBox.Image = null;

            this.textBox.Clear();

            this.keyTextBox.Clear();

            this.encryptButton.Enabled = false;
            this.decryptButton.Enabled = false;
        }

        #endregion
    }
}